<?php
namespace Ep\Woo\Email;

/**
 * Šalje se korisniku kada mu se dodijeli kupon s popustom.
 *
 * @package Ak\Woo\Email
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link http://gitlab.ep.local/autokreso/woo-coupon
 */
class CouponAdded extends \WC_Email
{
    /** @var string */
    protected $email_type = 'html';

    /**
     * Konstruktor.
     */
    public function __construct()
    {
        $this->id             = 'coupon_added';
        $this->customer_email = true;
        $this->title          = 'Kupon dodan';
        $this->description    = 'Korisniku se šalje poruka kada mu se dodijeli kupon s popustom';
        $this->template_html  = 'emails/customer-coupon-added.php';
        $this->placeholders   = array(
            '{site_title}'   => $this->get_blogname(),
            '{coupon_code}'   => '',
        );

        parent::__construct();

        $this->email_type = 'html';
    }

    /**
     * Get email subject.
     *
     * @since  3.1.0
     * @return string
     */
    public function get_default_subject() {
        return 'Vaš kupon s popustom na {site_title}';
    }

    /**
     * Get email heading.
     *
     * @since  3.1.0
     * @return string
     */
    public function get_default_heading() {
        return 'Vaš kupon s popustom';
    }

    /**
     * Initialise settings form fields.
     */
    public function init_form_fields()
    {
        $this->form_fields = [
            'enabled' => [
                'title' => 'Omogući/Onemogući',
                'type' => 'checkbox',
                'label' => 'Omogući ovu e-mail obavijest',
                'default' => 'yes',
            ],

        ];
    }

    /**
     * @param string $user_email
     * @param \WC_Coupon $coupon
     */
    public function trigger(string $user_email, \WC_Coupon $coupon)
    {
        if ($this->is_enabled()) {
            /** @noinspection PhpMethodParametersCountMismatchInspection */
            $this->send(
                $user_email,
                $this->get_subject(),
                $this->get_content_html([
                    'coupon' => $coupon,
                    'email' => $user_email,
                ]),
                $this->get_headers(),
                (string) $this->get_attachments()
            );
        }
    }

    /**
     * Get content html.
     *
     * @access public
     * @param \WC_Coupon|null $coupon
     *
     * @return string
     */
    public function get_content_html(array $args = []): string
    {
        $content = wc_get_template_html($this->template_html, [
            'email_heading'      => $this->get_heading(),
            'coupon'             => $args['coupon'],
            'plain_text'         => false,
            'email'              => $args['email'],
        ]);

        return $content;
    }
}