<?php
namespace Ep\Woo\Plugin\Checkout;

use Ep\View\Renderer\RendererAwareInterface;
use Ep\View\Renderer\RendererAwareTrait;
use Ak\Wp\Plugin\PluginInterface;

/**
 * Class R1
 *
 * @package Ep\Woo\Plugin\Checkout
 * @author Elektronika Perhoč <support@ep.hr>
 * @link https://bitbucket.org/elektronika-perhoc/ep-woocommerce
 */
class R1 implements
    PluginInterface,
    RendererAwareInterface
{
    use RendererAwareTrait;

    /**
     * Dodaje akcije i filtere u WordPress.
     */
    public function attach(): void
    {
        add_action('manage_edit-shop_order_columns', [$this, 'manage_edit_shop_order_columns'], 20);
        add_action('manage_shop_order_posts_custom_column', [$this, 'manage_shop_order_posts_custom_column'], 2);
        add_action('woocommerce_admin_order_data_after_billing_address', [$this, 'wc_admin_order_display_r1']);
        add_action('woocommerce_order_details_after_order_table', [$this, 'wc_admin_order_display_r1']);
        add_action('woocommerce_after_checkout_billing_form', [$this, 'wc_after_checkout_billing_form']);
        add_action('woocommerce_after_checkout_validation', [$this, 'wc_after_checkout_validation'], 100, 2);
        add_filter('woocommerce_checkout_fields', [$this, 'wc_checkout_fields']);
        add_action('woocommerce_checkout_update_order_meta', [$this, 'wc_checkout_update_order_meta']);
    }

    /**
     * Dodaje polje u header popisa narudbži u adminu.
     *
     * @param $columns
     *
     * @return array
     */
    public function manage_edit_shop_order_columns($columns)
    {
        $offset = 9;
        $updated_columns = array_slice($columns, 0, $offset, true) +
                           ['r1_checkbox' => esc_html(__('R1 račun', 'ep-woocommerce'))] +
                           array_slice($columns, $offset, null, true);

        return $updated_columns;
    }

    /**
     * Ispisuje vrijednost u popis narudžbi u adminu.
     *
     * @param $column
     */
    public function manage_shop_order_posts_custom_column($column)
    {
        global $post;

        if ($column === 'r1_checkbox') {
            echo '1' === get_post_meta($post->ID, 'R1 račun', true) ? 'Da' : '-';
        }
    }

    /**
     * Prikazuje R1 podatke na stranici narudžbe u adminu.
     *
     * @param \WC_Order $order
     */
    public function wc_admin_order_display_r1(\WC_Order $order)
    {
        if ('1' === get_post_meta($order->get_id(), 'R1 račun', true)) {
            printf('<h4>%s</h4>', __('R1 račun', 'ep-woocommerce'));
            printf('<p><strong>%s:</strong> %s</p>', __('Ime tvrtke', 'ep-woocommerce'), get_post_meta($order->get_id(), 'Ime tvrtke', true));
            printf('<p><strong>%s:</strong> %s</p>', __('Adresa tvrtke', 'ep-woocommerce'), get_post_meta($order->get_id(), 'Adresa tvrke', true));
            printf('<p><strong>%s:</strong> %s</p>', __('OIB tvrtke', 'ep-woocommerce'), get_post_meta($order->get_id(), 'OIB tvrtke', true));
            printf('<p><strong>%s:</strong> %s</p>', __('Poštanski broj tvrtke', 'ep-woocommerce'), get_post_meta($order->get_id(), 'Poštanski broj tvrtke', true));
            printf('<p><strong>%s:</strong> %s</p>', __('Grad tvrtke', 'ep-woocommerce'), get_post_meta($order->get_id(), 'Grad tvrtke', true));
        }
    }

    /**
     * Forma za unos podataka tvrtke.
     *
     * @param \WC_Checkout $checkout
     */
    public function wc_after_checkout_billing_form(\WC_Checkout $checkout)
    {
        ob_start();

        woocommerce_form_field('r1_checkbox', array(
            'type' => 'checkbox',
            'class' => array('r1-checkbox form-row-wide'),
            'label' => __('Da', 'ep-woocommerce'),
        ), $checkout->get_value('r1_checkbox'));

        woocommerce_form_field('r1_ime_tvrtke', array(
            'type' => 'text',
            'class' => array('r1-text form-row-wide'),
            'label' => __('Ime tvrtke', 'ep-woocommerce'),
            'placeholder' => _x('Upišite ime tvrtke', 'placeholder', 'ep-woocommerce'),
            'required' => true,
        ), $checkout->get_value('r1_ime_tvrtke'));

        woocommerce_form_field('r1_adresa_tvrtke', array(
            'type' => 'text',
            'class' => array('r1-text form-row-wide'),
            'label' => __('Adresa tvrtke', 'ep-woocommerce'),
            'placeholder' => _x('Upišite adresu tvrtke', 'placeholder', 'ep-woocommerce'),
            'required' => true,
        ), $checkout->get_value('r1_adresa_tvrtke'));

        woocommerce_form_field('r1_postanski_broj_tvrtke', array(
            'type' => 'text',
            'class' => array('r1-text form-row-first'),
            'label' => __('Poštanski broj', 'ep-woocommerce'),
            'placeholder' => _x('Upišite poštanski broj', 'placeholder', 'ep-woocommerce'),
            'required' => true,
        ), $checkout->get_value('r1_postanski_broj_tvrtke'));

        woocommerce_form_field('r1_grad_tvrtke', array(
            'type' => 'text',
            'class' => array('r1-text form-row-last'),
            'label' => __('Grad', 'ep-woocommerce'),
            'placeholder' => _x('Upišite grad', 'placeholder', 'ep-woocommerce'),
            'required' => true,
        ), $checkout->get_value('r1_grad_tvrtke'));

        woocommerce_form_field('r1_oib_tvrtke', array(
            'type' => 'text',
            'class' => array('r1-text form-row-wide'),
            'label' => __('OIB tvrtke', 'ep-woocommerce'),
            'placeholder' => _x('Upišite OIB tvrtke', 'placeholder', 'ep-woocommerce'),
            'required' => true,
        ), $checkout->get_value('r1_oib_tvrtke'));

        echo $this->renderer->render('@woo/checkout/r1.html.twig', [
            'title' => __('Pravne osobe', 'ep-woocommerce'),
            'text' => __('Trebate li R1 račun?', 'ep-woocommerce'),
            'form' => ob_get_clean(),
        ]);
    }

    /**
     * Nakon što WooCommerce odvalidira svoje, provjeravamo podatke potrebne za R1 račun.
     *
     * @param $data
     * @param \WP_Error $errors
     */
    public function wc_after_checkout_validation(/** @noinspection PhpUnusedParameterInspection */ $data, \WP_Error $errors)
    {
        if (isset($_POST['r1_checkbox']) && '1' === $_POST['r1_checkbox']) {
            $required = [
                'r1_ime_tvrtke' => __('Ime tvrtke', 'ep-woocommerce'),
                'r1_adresa_tvrtke' => __('Adresa tvrtke', 'ep-woocommerce'),
                'r1_postanski_broj_tvrtke' => __('Poštanski broj tvrtke', 'ep-woocommerce'),
                'r1_grad_tvrtke' => __('Grad tvrtke', 'ep-woocommerce'),
                'r1_oib_tvrtke' => __('OIB tvrtke', 'ep-woocommerce'),
            ];

            foreach ($required as $name => $label) {
                $value = isset($_POST[$name]) ? trim($_POST[$name]) : '';

                if (empty($value)) {
                    $errors->add('required-field', sprintf(__('<strong>%s</strong> je obavezno polje.', 'ep-woocommerce'), $label));
                }
            }
        }
    }

    /**
     * Uklanja polje za poduzeće, to tražimo za R1.
     *
     * @param array $fields
     *
     * @return array
     */
    public function wc_checkout_fields(array $fields)
    {
        unset($fields['billing']['billing_company']);
        return $fields;
    }

    /**
     * Dodaje R1 podatke u narudbžu.
     *
     * @param $order_id
     */
    public function wc_checkout_update_order_meta($order_id)
    {
        if (isset($_POST['r1_checkbox']) && '1' === $_POST['r1_checkbox']) {
            update_post_meta($order_id, 'R1 račun', esc_attr($_POST['r1_checkbox']));

            if (isset($_POST['r1_ime_tvrtke'])) {
                update_post_meta($order_id, 'Ime tvrtke', esc_attr($_POST['r1_ime_tvrtke']));
            }

            if (isset($_POST['r1_adresa_tvrtke'])) {
                update_post_meta($order_id, 'Adresa tvrtke', esc_attr($_POST['r1_adresa_tvrtke']));
            }

            if (isset($_POST['r1_oib_tvrtke'])) {
                update_post_meta($order_id, 'OIB tvrtke', esc_attr($_POST['r1_oib_tvrtke']));
            }

            if (isset($_POST['r1_postanski_broj_tvrtke'])) {
                update_post_meta($order_id, 'Poštanski broj tvrtke', esc_attr($_POST['r1_postanski_broj_tvrtke']));
            }

            if ($_POST['r1_grad_tvrtke']) {
                update_post_meta($order_id, 'Grad tvrtke', esc_attr($_POST['r1_grad_tvrtke']));
            }
        }
    }

    /**
     * Dodaje R1 podatke u mailove.
     *
     * @param array $keys
     *
     * @return array
     */
    public function wc_email_order_meta_keys(array $keys)
    {
        if (isset($_POST['r1_checkbox']) && '1' === $_POST['r1_checkbox']) {
            echo '<h2>R1 račun za kupca</h2>';

            $keys[] = 'Ime tvrtke';
            $keys[] = 'Adresa tvrtke';
            $keys[] = 'OIB tvrtke';
            $keys[] = 'Poštanski broj tvrtke';
            $keys[] = 'Grad tvrtke';
        }

        return $keys;
    }
}