<?php
namespace Ep\Woo\View;

use Ep\Gath\Gath;

class BrandFilter extends AbstractView
{
    protected $gath;

    public function __construct(Gath $gath = null)
    {
        $this->gath = $gath;
    }

    protected $templateOverrides = [
        'brand-filter.php',
    ];

    public function render(): string
    {
        if (null === $this->gath) {
            return '';
        }

        $brands = $this->gath->getBrandsByCategory( get_queried_object_id() );
        $brands = $brands['Results'];

        if (is_array($brands) && 0 === count($brands)) {
            return '';
        }

        $currentBrand = intval($_GET['brands'] ?? false);

        $active = [];
        $inactive = [];

        foreach ($brands as $brand) {
            $isActive = $brand['id'] === $currentBrand;

            $baseUrl = preg_replace('#/page/\d+/?#', '', $_SERVER['REQUEST_URI']);
            $baseUrl = preg_replace('#\?.+$#', '', $baseUrl);

            $item = [
                'active' => $isActive,
                'url' => $baseUrl . '?brands=' . ($isActive ? '' : $brand['id']),
                'name' => $brand['name'],
                'image' => rtrim($this->gath->getImageBaseUri(), '/') . '/' . ltrim($brand['image'], '/'),
            ];

            if ($isActive) {
                $active[] = $item;
            } else {
                $inactive[] = $item;
            }
        }

        $viewBrands = array_merge($active, $inactive);

        return $this->getRenderer()->render('@woo/brand-filter.html.twig', [
            'brands' => $viewBrands,
        ]);
    }
}
